#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""Tests for `cellmaps_pipeline` package."""
import unittest
import os
import shutil
import tempfile
from unittest.mock import patch, MagicMock

from cellmaps_utils import constants

from cellmaps_pipeline.runner import PipelineRunner
from cellmaps_pipeline.exceptions import CellmapsPipelineError


class TestPipelineRunner(unittest.TestCase):

    def setUp(self):
        pass

    def tearDown(self):
        pass

    def test_run(self):
        try:
            runner = PipelineRunner('foo')
            runner.run()
            self.fail('Expected NotImplementedError')
        except NotImplementedError as e:
            self.assertTrue('subclasses need to implement' in str(e))

    def test_get_image_coembed_tuples_fold_is_none(self):
        runner = PipelineRunner('foo')
        try:
            runner._get_image_coembed_tuples(None)
            self.fail('Expected CellmapsPipelineError')
        except CellmapsPipelineError as e:
            self.assertTrue('Fold cannot be None' in str(e))

    def test_get_image_coembed_tuples_fold_single_entry(self):
        temp_dir = tempfile.mkdtemp()
        try:
            runner = PipelineRunner(temp_dir)
            coembed_tuples = runner._get_image_coembed_tuples([1])
            self.assertEqual(1, coembed_tuples[0][0])
            self.assertEqual(os.path.join(temp_dir, constants.IMAGE_EMBEDDING_STEP_DIR + '1'),
                             coembed_tuples[0][1])
            self.assertEqual(os.path.join(temp_dir, constants.COEMBEDDING_STEP_DIR + '1'),
                             coembed_tuples[0][2])
        finally:
            shutil.rmtree(temp_dir)

    def test_get_image_coembed_tuples_fold_multiple_entry(self):
        temp_dir = tempfile.mkdtemp()
        try:
            runner = PipelineRunner(temp_dir)
            coembed_tuples = runner._get_image_coembed_tuples([1, 2])
            for entry in [1, 2]:
                print(coembed_tuples)
                self.assertEqual(entry, coembed_tuples[entry-1][0])
                self.assertEqual(os.path.join(temp_dir,
                                              constants.IMAGE_EMBEDDING_STEP_DIR +
                                              str(entry)),
                                 coembed_tuples[entry-1][1])
                self.assertEqual(os.path.join(temp_dir,
                                              constants.COEMBEDDING_STEP_DIR +
                                              str(entry)),
                                 coembed_tuples[entry-1][2])
        finally:
            shutil.rmtree(temp_dir)


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""Tests for `cellmaps_pipeline` package."""
import unittest

from cellmaps_imagedownloader.exceptions import CellMapsImageDownloaderError

from cellmaps_pipeline.runner import ProgrammaticPipelineRunner
import os
import shutil
from unittest.mock import patch, MagicMock


class TestProgrammaticPipelineRunner(unittest.TestCase):

    def setUp(self):
        self.outdir = './temp_dir'
        self.runner = ProgrammaticPipelineRunner(outdir=self.outdir)

    def tearDown(self):
        if os.path.exists(self.outdir):
            shutil.rmtree(self.outdir)

    @patch("os.path.isdir", return_value=False)
    def test_image_download_missing_parameters(self, mock_isdir):
        self.runner._cm4ai_image = None
        self.runner._samples = None
        self.runner._unique = None
        with self.assertRaises(CellMapsImageDownloaderError):
            self.runner._download_images()


if __name__ == '__main__':
    unittest.main()



#! /usr/bin/env python

import os
import warnings
import logging
import time
import networkx as nx
from cellmaps_imagedownloader.exceptions import CellMapsImageDownloaderError
from cellmaps_utils import logutils
from cellmaps_utils import constants
from cellmaps_utils.provenance import ProvenanceUtil

from cellmaps_imagedownloader.runner import MultiProcessImageDownloader
from cellmaps_imagedownloader.runner import FakeImageDownloader
from cellmaps_imagedownloader.runner import CellmapsImageDownloader
from cellmaps_imagedownloader.runner import CM4AICopyDownloader
from cellmaps_imagedownloader.gene import ImageGeneNodeAttributeGenerator, CM4AITableConverter
from cellmaps_ppidownloader.runner import CellmapsPPIDownloader
from cellmaps_ppidownloader.gene import APMSGeneNodeAttributeGenerator, CM4AIGeneNodeAttributeGenerator
from cellmaps_ppi_embedding.runner import Node2VecEmbeddingGenerator
from cellmaps_ppi_embedding.runner import CellMapsPPIEmbedder
from cellmaps_image_embedding.runner import CellmapsImageEmbedder
from cellmaps_image_embedding.runner import FakeEmbeddingGenerator
from cellmaps_image_embedding.runner import DensenetEmbeddingGenerator
from cellmaps_coembedding.runner import MuseCoEmbeddingGenerator
from cellmaps_coembedding.runner import FakeCoEmbeddingGenerator
from cellmaps_coembedding.runner import CellmapsCoEmbedder
from cellmaps_generate_hierarchy.ppi import CosineSimilarityPPIGenerator
from cellmaps_generate_hierarchy.hierarchy import CDAPSHiDeFHierarchyGenerator
from cellmaps_generate_hierarchy.maturehierarchy import HiDeFHierarchyRefiner
from cellmaps_generate_hierarchy.runner import CellmapsGenerateHierarchy
from cellmaps_generate_hierarchy.hcx import HCXFromCDAPSCXHierarchy
from cellmaps_imagedownloader.proteinatlas import ProteinAtlasReader, CM4AIImageCopyTupleGenerator
from cellmaps_imagedownloader.proteinatlas import ProteinAtlasImageUrlReader
from cellmaps_imagedownloader.proteinatlas import ImageDownloadTupleGenerator
from cellmaps_imagedownloader.proteinatlas import LinkPrefixImageDownloadTupleGenerator
from cellmaps_hierarchyeval.runner import CellmapshierarchyevalRunner

import cellmaps_pipeline
from cellmaps_pipeline.exceptions import CellmapsPipelineError

logger = logging.getLogger(__name__)


class PipelineRunner(object):
    """
    Base class for running pipeline commands in a generic execution environment.
    This class should be subclassed to provide specific implementations for different
    execution environments such as local or SLURM-based clusters.
    """

    def __init__(self, outdir):
        """
        Constructor

        :param outdir: The output directory where all pipeline generated files will be stored.
        """
        self._outdir = os.path.abspath(outdir)

    def _get_image_coembed_tuples(self, fold):
        """
        Generate tuples containing fold number and directories for image embedding and coembedding.

        :param fold: List of integers representing the folds of the data.
        :return: A list of tuples, each containing the fold number, image embedding directory,
                 and coembedding directory paths.
        """
        if fold is None:
            raise CellmapsPipelineError('Fold cannot be None')

        image_coembed_tuples = []
        logger.debug('Fold values: ' + str(fold))
        for fold_val in fold:
            image_embed_dir = os.path.join(self._outdir,
                                           constants.IMAGE_EMBEDDING_STEP_DIR +
                                           str(fold_val))
            co_embed_dir = os.path.join(self._outdir,
                                        constants.COEMBEDDING_STEP_DIR +
                                        str(fold_val))

            image_coembed_tuples.append((fold_val, image_embed_dir,
                                         co_embed_dir))
        logger.debug('Value of image_coembed_tuples: ' +
                     str(image_coembed_tuples))
        return image_coembed_tuples

    def run(self):
        """
        Abstract method to run the pipeline. This method should be implemented by subclasses.

        :raises NotImplementedError: If the subclass does not implement this method.
        """
        raise NotImplementedError('subclasses need to implement')


class SLURMPipelineRunner(PipelineRunner):
    """
    Generates SLURM batch files and wrapper script to
    run various steps in a SLURM environment
    """

    def __init__(self, outdir=None,
                 cm4ai_apms=None,
                 cm4ai_image=None,
                 samples=None,
                 unique=None,
                 edgelist=None,
                 baitlist=None,
                 model_path=None,
                 proteinatlasxml=None,
                 ppi_cutoffs=None,
                 fake=None,
                 provenance=None,
                 fold=[1],
                 input_data_dict=None,
                 slurm_partition=None,
                 slurm_account=None):
        """
        :param outdir: Path to the output directory.
        :param cm4ai_apms: Path to the CM4AI APMS data file.
        :param cm4ai_image: Path to the CM4AI image data file.
        :param samples: Path to the samples data file.
        :param unique: Path to the unique data file.
        :param edgelist: Path to the edge list file for PPI data.
        :param baitlist: Path to the bait list file for PPI data.
        :param model_path: Path to the pre-trained model for embedding generation.
        :param proteinatlasxml: Path to the Protein Atlas XML data.
        :param ppi_cutoffs: Cutoff thresholds for PPI data filtering.
        :param fake: Boolean indicating whether to use fake data embedding for testing.
        :param provenance: Path to the provenance data file.
        :param fold: Data folds to process.
        :param input_data_dict: Dictionary of input data configurations.
        :param slurm_partition: Name of the SLURM partition to submit jobs to.
        :param slurm_account: SLURM account name for job submission.
        """
        super().__init__(outdir=outdir)
        self._cm4ai_apms = cm4ai_apms if cm4ai_apms is None or os.path.isabs(cm4ai_apms) else os.path.join(os.getcwd(),
                                                                                                           cm4ai_apms)
        self._cm4ai_image = cm4ai_image if cm4ai_image is None or os.path.isabs(cm4ai_image) else os.path.join(
            os.getcwd(), cm4ai_image)
        self._samples = samples if samples is None or os.path.isabs(samples) else os.path.join(os.getcwd(), samples)
        self._unique = unique if unique is None or os.path.isabs(unique) else os.path.join(os.getcwd(), unique)
        self._edgelist = edgelist if edgelist is None or os.path.isabs(edgelist) else os.path.join(os.getcwd(),
                                                                                                   edgelist)
        self._baitlist = baitlist if baitlist is None or os.path.isabs(baitlist) else os.path.join(os.getcwd(),
                                                                                                   baitlist)
        self._model_path = model_path
        self._fake = fake
        self._provenance = provenance if provenance is None or os.path.isabs(provenance) else os.path.join(os.getcwd(),
                                                                                                           provenance)
        self._proteinatlasxml = proteinatlasxml
        self._ppi_cutoffs = ppi_cutoffs
        self._input_data_dict = input_data_dict
        self._slurm_partition = slurm_partition
        self._slurm_account = slurm_account
        self._image_dir = os.path.join(self._outdir,
                                       constants.IMAGE_DOWNLOAD_STEP_DIR)
        self._ppi_dir = os.path.join(self._outdir,
                                     constants.PPI_DOWNLOAD_STEP_DIR)
        self._ppi_embed_dir = os.path.join(self._outdir,
                                           constants.PPI_EMBEDDING_STEP_DIR)

        self._image_coembed_tuples = self._get_image_coembed_tuples(fold)

        self._hierarchy_dir = os.path.join(self._outdir,
                                           constants.HIERARCHY_STEP_DIR)

        self._hierarchy_eval_dir = os.path.join(self._outdir,
                                                constants.HIERARCHYEVAL_STEP_DIR)

    def _write_slurm_directives(self, out=None,
                                allocated_time='4:00:00',
                                mem='32G', cpus_per_task='4',
                                job_name='cellmaps_pipeline'):
        """
        Writes SLURM job directives to a bash script file.

        :param out: File handle to write the SLURM directives.
        :param allocated_time: String specifying the maximum time allowed for the job.
        :param mem: String specifying the memory allocated for the job.
        :param cpus_per_task: String specifying the number of CPUs per task.
        :param job_name: String specifying the name of the SLURM job.
        """
        out.write('#!/bin/bash\n\n')
        out.write('#SBATCH --job-name=' + str(job_name) + '\n')
        out.write('#SBATCH --chdir=' + self._outdir + '\n')

        out.write('#SBATCH --output=%x.%j.out\n')
        if self._slurm_partition is not None:
            out.write('#SBATCH --partition=' + self._slurm_partition + '\n')
        if self._slurm_account is not None:
            out.write('#SBATCH --account=' + self._slurm_account + '\n')
        out.write('#SBATCH --ntasks=1\n')
        out.write('#SBATCH --cpus-per-task=' + str(cpus_per_task) + '\n')
        out.write('#SBATCH --mem=' + str(mem) + '\n')
        out.write('#SBATCH --time=' + str(allocated_time) + '\n\n')

        out.write('echo $SLURM_JOB_ID\n')
        out.write('echo $HOSTNAME\n')

    def _write_directory_check(self, out, directory):
        """
        Writes a directory check script to the provided file handle, which exits the job if the directory exists.

        :param out: File handle to write the directory check script.
        :param directory: The directory to check for existence.
        """
        out.write(f"if [ -d \"{directory}\" ]; then\n")
        out.write(f"    echo \"Directory {directory} exists. Skipping job.\"\n")
        out.write("    exit 0\n")
        out.write("fi\n\n")

    def _generate_download_images_command(self):
        """
        Generates a bash script for downloading images and writes it to a file in the output directory.

        :return: The filename of the bash script generated for downloading images.
        """
        with open(os.path.join(self._outdir, 'imagedownloadjob.sh'), 'w') as f:
            self._write_slurm_directives(out=f, job_name='imagedownload')
            self._write_directory_check(f, self._image_dir)
            if self._cm4ai_image != None:
                input_arg = '--cm4ai_table ' + self._cm4ai_image
            elif self._samples != None and self._unique != None:
                input_arg = '--samples ' + self._samples + ' --unique ' + self._unique
            else:
                raise CellmapsPipelineError(
                    'You must provide cm4ai_table parameter or samples and unque parameters.')
            if self._provenance == None:
                raise CellmapsPipelineError(
                    'You must provide provenance parameter')
            f.write('cellmaps_imagedownloadercmd.py ' + self._image_dir +
                    ' --provenance ' + self._provenance + ' ' + input_arg + '\n')
            f.write('exit $?\n')
        os.chmod(os.path.join(self._outdir, 'imagedownloadjob.sh'), 0o755)
        return 'imagedownloadjob.sh'

    def _generate_download_ppi_command(self):
        """
        Generates a bash script for downloading protein-protein interactions (PPI) data and writes it to a file in the output directory.

        :return: ppidownloadjob.sh
        """
        with open(os.path.join(self._outdir, 'ppidownloadjob.sh'), 'w') as f:
            self._write_slurm_directives(out=f, job_name='ppidownload')
            self._write_directory_check(f, self._ppi_dir)
            if self._cm4ai_apms != None:
                input_arg = '--cm4ai_table ' + self._cm4ai_apms
            elif self._edgelist != None and self._baitlist != None:
                input_arg = '--edgelist ' + self._edgelist + ' --baitlist ' + self._baitlist
            else:
                raise CellmapsPipelineError(
                    'You must provide edgelist and baitlist parameters.')
            if self._provenance == None:
                raise CellmapsPipelineError(
                    'You must provide provenance parameter')
            f.write('cellmaps_ppidownloadercmd.py ' + self._ppi_dir +
                    ' --provenance ' + self._provenance + ' ' + input_arg + '\n')
            f.write('exit $?\n')
        os.chmod(os.path.join(self._outdir, 'ppidownloadjob.sh'), 0o755)
        return 'ppidownloadjob.sh'

    def _generate_embed_image_command(self, fold=1):
        """
        Generates a bash script for embedding images based on the specified fold and writes it to a file in the output directory.

        :param fold: The data fold to process for image embedding.
        :return: The filename of the bash script generated for image embedding.
        """
        filename = 'imageembedjob' + str(fold) + '.sh'
        with open(os.path.join(self._outdir, filename), 'w') as f:
            self._write_slurm_directives(out=f, job_name='imageembed' + str(fold))
            self._write_directory_check(f, self._image_coembed_tuples[fold - 1][1])
            fake = '--fake_embedder' if self._fake is True else ""
            f.write('cellmaps_image_embeddingcmd.py ' + self._image_coembed_tuples[fold - 1][1] +
                    ' --fold ' + str(fold) + ' --inputdir ' + self._image_dir + ' ' + fake + ' -vvvv\n')
            f.write('exit $?\n')
        os.chmod(os.path.join(self._outdir, filename), 0o755)
        return filename

    def _generate_embed_ppi_command(self):
        """
        Generates a bash script for embedding PPI data and writes it to a file in the output directory.

        :return: ppiembedjob.sh
        """
        with open(os.path.join(self._outdir, 'ppiembedjob.sh'), 'w') as f:
            self._write_slurm_directives(out=f, job_name='ppiembed')
            self._write_directory_check(f, self._ppi_embed_dir)
            fake = "--fake_embedder" if self._fake is True else ""
            f.write('cellmaps_ppi_embeddingcmd.py ' + self._ppi_embed_dir +
                    ' --inputdir ' + self._ppi_dir + ' ' + fake + ' -vvvv\n')
            f.write('exit $?\n')
        os.chmod(os.path.join(self._outdir, 'ppiembedjob.sh'), 0o755)
        return 'ppiembedjob.sh'

    def _generate_coembed_command(self, fold=1):
        """
        Generates a bash script for co-embedding of image and PPI data based on the specified fold and writes it to a file in the output directory.

        :param fold: The data fold to process for co-embedding.
        :return: coembedjob.sh
        """
        filename = 'coembeddingjob' + str(fold) + '.sh'
        with open(os.path.join(self._outdir, filename), 'w') as f:
            self._write_slurm_directives(out=f, job_name='coembedding' + str(fold))
            self._write_directory_check(f, self._image_coembed_tuples[fold - 1][2])
            fake = '--fake_embedding' if self._fake is True else ""
            f.write('cellmaps_coembeddingcmd.py ' + self._image_coembed_tuples[fold - 1][
                2] + ' --ppi_embeddingdir ' + self._ppi_embed_dir +
                    ' --image_embeddingdir ' + self._image_coembed_tuples[fold - 1][1] + ' ' + fake + ' -vvvv\n')
            f.write('exit $?\n')
        os.chmod(os.path.join(self._outdir, filename), 0o755)
        return filename

    def _generate_hierarchy_command(self):
        """
        Generates a bash script for constructing a hierarchy from the co-embedded data.
        :return: hierarchyjob.sh
        """
        with open(os.path.join(self._outdir, 'hierarchyjob.sh'), 'w') as f:
            self._write_slurm_directives(out=f, job_name='hierarchy')
            self._write_directory_check(f, self._hierarchy_dir)
            f.write('cellmaps_generate_hierarchycmd.py ' + self._hierarchy_dir + ' --coembedding_dirs ')
            for image_coembed_tuple in self._image_coembed_tuples:
                f.write(image_coembed_tuple[2] + ' ')
            f.write('--gene_node_attributes ' + self._ppi_dir + ' ' + self._image_dir + ' ')
            f.write('-vvvv\n')
            f.write('exit $?\n')
        os.chmod(os.path.join(self._outdir, 'hierarchyjob.sh'), 0o755)
        return 'hierarchyjob.sh'

    def _generate_hierarchyeval_command(self):
        """
        Generates a bash script for evaluating the generated hierarchy.
        :return: hierarchyevaljob.sh
        """
        with open(os.path.join(self._outdir, 'hierarchyevaljob.sh'), 'w') as f:
            self._write_slurm_directives(out=f, job_name='hierarchyeval')
            self._write_directory_check(f, self._hierarchy_eval_dir)
            f.write('cellmaps_hierarchyevalcmd.py ' + self._hierarchy_eval_dir + ' --hierarchy_dir ' +
                    self._hierarchy_dir)
            f.write(' -vvvv\n')
            f.write('exit $?\n')
        os.chmod(os.path.join(self._outdir, 'hierarchyevaljob.sh'), 0o755)
        return 'hierarchyevaljob.sh'

    def run(self):
        """
        Runs pipelines
        """
        slurmjobfile = os.path.join(self._outdir, 'slurm_cellmaps_job.sh')
        with open(slurmjobfile, 'w') as f:
            f.write('#! /bin/bash\n\n')
            f.write('# image download no dependencies\n')
            f.write('image_download_job=$(sbatch ' +
                    self._generate_download_images_command() + ' | awk \'{print $4}\')\n\n')

            f.write('# ppi download no dependencies\n')
            f.write('ppi_download_job=$(sbatch ' +
                    self._generate_download_ppi_command() + ' | awk \'{print $4}\')\n\n')

            f.write('# ppi embed\n')
            f.write('ppi_embed_job=$(sbatch --dependency=afterok:$ppi_download_job ' +
                    self._generate_embed_ppi_command() + ' | awk \'{print $4}\')\n\n')

            embed_job_names = ['$ppi_embed_job']
            for image_coembed_tuple in self._image_coembed_tuples:
                # [0] = fold value
                # [1] = image embedding dir
                # [2] = outdir
                f.write('# image embed\n')
                f.write('image_embed_job' + str(
                    image_coembed_tuple[0]) + '=$(sbatch --dependency=afterok:$image_download_job ' +
                        self._generate_embed_image_command(fold=image_coembed_tuple[0]) + ' | awk \'{print $4}\')\n\n')
                f.write(
                    '# fold' + str(image_coembed_tuple[0]) + ' co-embedding\n')
                embed_job_name = 'f' + str(image_coembed_tuple[0]) + '_coembed_job'
                f.write(embed_job_name + '=$(sbatch --dependency=afterok:$image_embed_job' + str(
                    image_coembed_tuple[0]) + ' ' +
                        self._generate_coembed_command(fold=image_coembed_tuple[0]) + ' | awk \'{print $4}\')\n\n')
                embed_job_names.append('$' + embed_job_name)
            dependency_str = ':'.join(embed_job_names)
            f.write('# hierarchy\n')
            f.write(
                'hierarchy_job=$(sbatch --dependency=afterok:' + dependency_str + ' ' + self._generate_hierarchy_command() + ' | awk \'{print $4}\')\n\n')
            f.write('echo "job submitted; here is ID of final hierarchy job: $hierarchy_job"\n')
            f.write('# hierarchyeval\n')
            f.write(
                'hierarchyeval_job=$(sbatch --dependency=afterok:$hierarchy_job ' +
                self._generate_hierarchyeval_command() + ' | awk \'{print $4}\')\n\n')
            f.write('echo "job submitted; here is ID of final hierarchyeval job: $hierarchyeval_job"\n')
        os.chmod(slurmjobfile, 0o755)


class ProgrammaticPipelineRunner(PipelineRunner):
    """
    Runs pipeline programmatically in a serial fashion

    """

    def __init__(self, outdir=None,
                 cm4ai_apms=None,
                 cm4ai_image=None,
                 samples=None,
                 unique=None,
                 edgelist=None,
                 baitlist=None,
                 model_path=None,
                 proteinatlasxml=None,
                 ppi_cutoffs=None,
                 fake=None,
                 provenance=None,
                 skip_logging=False,
                 provenance_utils=ProvenanceUtil(),
                 fold=[1],
                 input_data_dict=None):
        """
        Constructor

        :param outdir: Output directory for results and logs.
        :param cm4ai_apms: Path to CM4AI AP-MS data.
        :param cm4ai_image: Path to CM4AI image data.
        :param samples: Path to samples data.
        :param unique: Path to unique data.
        :param edgelist: Path to the network edge list.
        :param baitlist: Path to the bait list.
        :param model_path: Path to the model used for embedding.
        :param proteinatlasxml: Path to the ProteinAtlas XML data.
        :param ppi_cutoffs: Cutoff thresholds for PPI data processing.
        :param fake: Uses fake embeddings for testing purposes.
        :param provenance: Provenance information for reproducibility.
        :param skip_logging: Skips logging of pipeline steps if True.
        :param provenance_utils: Utility for handling provenance data.
        :param fold: List of fold of image data.
        :param input_data_dict: Dictionary containing input data configurations.
        """
        super().__init__(outdir=outdir)
        self._cm4ai_amps = cm4ai_apms
        self._cm4ai_image = cm4ai_image
        self._samples = samples
        self._unique = unique
        self._edgelist = edgelist
        self._baitlist = baitlist
        self._model_path = model_path
        self._fake = fake
        self._provenance = provenance
        self._provenance_utils = provenance_utils
        self._proteinatlasxml = proteinatlasxml
        self._ppi_cutoffs = ppi_cutoffs
        self._input_data_dict = input_data_dict
        self._skip_logging = skip_logging
        self._image_dir = os.path.join(self._outdir,
                                       constants.IMAGE_DOWNLOAD_STEP_DIR)
        self._ppi_dir = os.path.join(self._outdir,
                                     constants.PPI_DOWNLOAD_STEP_DIR)
        self._ppi_embed_dir = os.path.join(self._outdir,
                                           constants.PPI_EMBEDDING_STEP_DIR)

        self._image_coembed_tuples = self._get_image_coembed_tuples(fold)

        self._hierarchy_dir = os.path.join(self._outdir,
                                           constants.HIERARCHY_STEP_DIR)

        self._hierarchy_eval_dir = os.path.join(self._outdir,
                                                constants.HIERARCHYEVAL_STEP_DIR)

    def run(self):
        """
        Runs pipeline programmatically in serial steps. This would
        be the same as running the steps in a notebook.

        :raises CellmapsPipelineError: If any step in the pipeline fails, indicating the step and reason.
        :return: Exit code 0 if successful, other values indicate failure.
        """
        if self._download_images() != 0:
            raise CellmapsPipelineError('Image download failed')

        if self._download_ppi() != 0:
            raise CellmapsPipelineError('PPI download failed')

        if self._embed_ppi() != 0:
            raise CellmapsPipelineError('PPI embed failed')

        if self._embed_image() != 0:
            raise CellmapsPipelineError('Image embed failed')

        if self._coembed() != 0:
            raise CellmapsPipelineError('Coembed failed')

        if self._hierarchy() != 0:
            raise CellmapsPipelineError('Hierarchy failed')

        if self._hierarchy_eval() != 0:
            raise CellmapsPipelineError('Hierarchy eval failed')

        return 0

    def _hierarchy_eval(self):
        """
        Evaluates the hierarchy.

        :return: Exit code 0 if successful, or if the directory already exists.
        """
        if os.path.isdir(self._hierarchy_eval_dir):
            warnings.warn(
                'Found hierarchy eval dir, assuming we are good. skipping')
            return 0

        return CellmapshierarchyevalRunner(outdir=self._hierarchy_eval_dir,
                                           hierarchy_dir=self._hierarchy_dir,
                                           input_data_dict=self._input_data_dict,
                                           provenance_utils=self._provenance_utils).run()

    def _hierarchy(self):
        """
        Generates a hierarchical structure of co-embedded data.

        :return: Exit code 0 if hierarchy generation is successful or skipped, otherwise logs an error.
        :rtype: int
        """
        if os.path.isdir(self._hierarchy_dir):
            warnings.warn(
                'Found hierarchy dir, assuming we are good. skipping')
            return 0

        coembed_dirs = []
        for image_coembed_tuple in self._image_coembed_tuples:
            coembed_dirs.append(image_coembed_tuple[2])

        logger.debug('Coembedding directories: ' + str(coembed_dirs))

        ppigen = CosineSimilarityPPIGenerator(embeddingdirs=coembed_dirs,
                                              cutoffs=self._ppi_cutoffs)

        refiner = HiDeFHierarchyRefiner(
            provenance_utils=self._provenance_utils)

        converter = HCXFromCDAPSCXHierarchy()

        hiergen = CDAPSHiDeFHierarchyGenerator(refiner=refiner,
                                               hcxconverter=converter,
                                               provenance_utils=self._provenance_utils)
        return CellmapsGenerateHierarchy(outdir=self._hierarchy_dir,
                                         inputdirs=coembed_dirs,
                                         ppigen=ppigen,
                                         gene_node_attributes=[self._image_dir, self._ppi_dir],
                                         hiergen=hiergen,
                                         skip_logging=self._skip_logging,
                                         input_data_dict=self._input_data_dict,
                                         provenance_utils=self._provenance_utils).run()

    def _coembed(self):
        """
        Performs co-embedding of image and PPI data, using either a real or fake data generator based on configuration.

        :return: Exit code 0 if co-embedding is successful or previously completed,
                 otherwise logs an error and returns non-zero.
        :rtype: int
        """
        for image_coembed_tuple in self._image_coembed_tuples:
            if os.path.isdir(image_coembed_tuple[2]):
                warnings.warn('Found coembedding dir' +
                              str(image_coembed_tuple[2]) +
                              ', assuming we are good. skipping')
                continue
            if self._fake:
                gen = FakeCoEmbeddingGenerator(ppi_embeddingdir=self._ppi_embed_dir,
                                               image_embeddingdir=image_coembed_tuple[1])
            else:
                gen = MuseCoEmbeddingGenerator(outdir=image_coembed_tuple[2],
                                               ppi_embeddingdir=self._ppi_embed_dir,
                                               image_embeddingdir=image_coembed_tuple[1])
            retval = CellmapsCoEmbedder(outdir=image_coembed_tuple[2],
                                        inputdirs=[image_coembed_tuple[1],
                                                   self._ppi_embed_dir],
                                        embedding_generator=gen,
                                        skip_logging=self._skip_logging,
                                        input_data_dict=self._input_data_dict).run()
            if retval != 0:
                logger.error('Coembedding ' + image_coembed_tuple[2] +
                             'using ' + image_coembed_tuple[1] +
                             ' had non zero exit code of: ' +
                             str(retval))
                return retval
        return 0

    def _embed_image(self):
        """
        Embeds image data using a specified model, typically a Densenet model.

        :return: Exit code 0 if the embedding is successful or already completed,
                 otherwise it logs an error and returns non-zero.
        :rtype: int
        """
        for image_coembed_tuple in self._image_coembed_tuples:
            if os.path.isdir(image_coembed_tuple[1]):
                warnings.warn('Found image_embedding dir' +
                              str(image_coembed_tuple[1]) +
                              ', assuming we are good. skipping')
                continue
            if self._fake is True:
                gen = FakeEmbeddingGenerator(self._image_dir)
            else:
                gen = DensenetEmbeddingGenerator(self._image_dir,
                                                 outdir=image_coembed_tuple[1],
                                                 model_path=self._model_path,
                                                 fold=int(image_coembed_tuple[0]))
            retval = CellmapsImageEmbedder(outdir=image_coembed_tuple[1],
                                           inputdir=self._image_dir,
                                           embedding_generator=gen,
                                           skip_logging=self._skip_logging,
                                           input_data_dict=self._input_data_dict).run()
            if retval != 0:
                logger.error('image embedding ' + image_coembed_tuple[1] +
                             'using fold' + str(image_coembed_tuple[0] +
                                                ' had non zero exit code of: ' +
                                                str(retval)))
                return retval
        return 0

    def _embed_ppi(self):
        """
        Embeds the protein-protein interaction data using the Node2Vec algorithm.

        :return: Exit code 0 if the directory exists or embedding is successful, otherwise logs an error.
        :rtype: int
        """
        if os.path.isdir(self._ppi_embed_dir):
            warnings.warn(
                'Found ppi embedding dir, assuming we are good. skipping')
            return 0
        gen = Node2VecEmbeddingGenerator(
            nx_network=nx.read_edgelist(CellMapsPPIEmbedder.get_apms_edgelist_file(self._ppi_dir),
                                        delimiter='\t'))

        return CellMapsPPIEmbedder(outdir=self._ppi_embed_dir,
                                   embedding_generator=gen,
                                   inputdir=self._ppi_dir,
                                   skip_logging=self._skip_logging,
                                   input_data_dict=self._input_data_dict).run()

    def _download_ppi(self):
        """
        Downloads protein-protein interaction (PPI) data. This method handles the determination of data sources
        and ensures that data is downloaded only once by checking the presence of the target directory.

        :return: Exit code 0 if the directory exists or download is successful,
                 otherwise it continues to handle or log the error.
        :rtype: int
        """
        if os.path.isdir(self._ppi_dir):
            warnings.warn('Found ppi dir, assuming we are good. skipping')
            return 0

        json_prov = self._provenance
        if self._cm4ai_amps is None:
            apmsgen = APMSGeneNodeAttributeGenerator(
                apms_edgelist=APMSGeneNodeAttributeGenerator.get_apms_edgelist_from_tsvfile(
                    self._edgelist),
                apms_baitlist=APMSGeneNodeAttributeGenerator.get_apms_baitlist_from_tsvfile(self._baitlist))
        else:
            json_prov[CellmapsPPIDownloader.CM4AI_ROCRATE] = os.path.abspath(os.path.dirname(self._cm4ai_amps))
            apmsgen = CM4AIGeneNodeAttributeGenerator(apms_edgelist=
                                                      CM4AIGeneNodeAttributeGenerator.get_apms_edgelist_from_tsvfile(
                                                          self._cm4ai_amps))

        return CellmapsPPIDownloader(outdir=self._ppi_dir,
                                     apmsgen=apmsgen,
                                     input_data_dict=self._input_data_dict,
                                     skip_logging=self._skip_logging,
                                     provenance=json_prov).run()

    def _download_images(self):
        """
        Downloads Images using
        :py:class:`~cellmaps_imagedownloader.runner.CellmapsImageDownloader`

        :return: exit code of :py:meth:`~cellmaps_imagedownloader.runner.CellmapsImageDownloader.run`
        :rtype: int
        """
        if os.path.isdir(self._image_dir):
            warnings.warn('Found image dir, assuming we are good. skipping')
            return 0
        logger.info('Downloading images')

        if self._cm4ai_image is not None:
            converter = CM4AITableConverter(cm4ai=self._cm4ai_image)
            samples_list, unique_list = converter.get_samples_and_unique_lists()
        elif self._samples is not None:
            samples_list = ImageGeneNodeAttributeGenerator.get_samples_from_csvfile(self._samples)
            unique_list = None
            if self._unique is not None:
                unique_list = ImageGeneNodeAttributeGenerator.get_unique_list_from_csvfile(self._unique)
        else:
            raise CellMapsImageDownloaderError("Parameters --samples and "
                                               "or --cm4ai_image must be "
                                               "provided.")

        imagegen = ImageGeneNodeAttributeGenerator(unique_list=unique_list,
                                                   samples_list=samples_list)

        if self._cm4ai_image is not None:
            imageurlgen = CM4AIImageCopyTupleGenerator(samples_list=imagegen.get_samples_list())
        elif 'linkprefix' in imagegen.get_samples_list()[0]:
            logger.debug(
                'linkprefix in samples using LinkPrefixImageDownloadTupleGenerator')
            imageurlgen = LinkPrefixImageDownloadTupleGenerator(
                samples_list=imagegen.get_samples_list())
        else:
            proteinatlas_reader = ProteinAtlasReader(
                self._image_dir, proteinatlas=self._proteinatlasxml)
            proteinatlas_urlreader = ProteinAtlasImageUrlReader(
                reader=proteinatlas_reader)
            imageurlgen = ImageDownloadTupleGenerator(reader=proteinatlas_urlreader,
                                                      samples_list=imagegen.get_samples_list())

        if self._cm4ai_image is not None:
            dloader = CM4AICopyDownloader()
        elif self._fake is True:
            warnings.warn('FAKE IMAGES ARE BEING DOWNLOADED!!!!!')
            dloader = FakeImageDownloader()
        else:
            dloader = MultiProcessImageDownloader()
        # Todo: input_data_dict should NOT be required to run this
        #       https://github.com/idekerlab/cellmaps_imagedownloader/issues/2
        return CellmapsImageDownloader(outdir=self._image_dir,
                                       imagedownloader=dloader,
                                       imagegen=imagegen,
                                       imageurlgen=imageurlgen,
                                       provenance=self._provenance,
                                       skip_failed=True,
                                       skip_logging=self._skip_logging,
                                       input_data_dict=self._input_data_dict).run()


class CellmapsPipeline(object):
    """
    Manages the execution of the Cellmaps pipeline. This class is responsible for setting up the environment,
    executing the runner, and handling the logging and cleanup tasks associated with the pipeline execution.
    """

    def __init__(self, outdir=None,
                 runner=None,
                 input_data_dict=None):
        """
        Constructor

        :param outdir: The directory where the pipeline's output will be stored.
        :param runner: The runner object responsible for executing the pipeline steps.
        :param input_data_dict: A dictionary of input data settings that may affect pipeline execution.
        :raises CellmapsPipelineError: If the output directory is not provided, it raises an error.
        """
        if outdir is None:
            raise CellmapsPipelineError('outdir is None')

        self._outdir = os.path.abspath(outdir)
        self._start_time = int(time.time())
        self._runner = runner
        self._input_data_dict = input_data_dict
        logger.debug('In constructor')

    def run(self):
        """
        Runs CM4AI Pipeline. This method ensures that all steps are logged and any
        exceptions are caught, and the final status is returned.

        :return: The exit status of the pipeline run. Returns 0 if successful, otherwise returns an error code.
        :rtype: int
        """
        logger.debug('In run method')

        if not os.path.isdir(self._outdir):
            os.makedirs(self._outdir, mode=0o755)

        logutils.write_task_start_json(outdir=self._outdir,
                                       start_time=self._start_time,
                                       data={
                                           'commandlineargs': self._input_data_dict},
                                       version=cellmaps_pipeline.__version__)

        exit_status = 99
        try:
            exit_status = self._runner.run()
        finally:
            logutils.write_task_finish_json(outdir=self._outdir,
                                            start_time=self._start_time,
                                            status=exit_status)
        return exit_status



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""Integration Tests for `cellmaps_pipeline` package."""

import os
import sys
import json
import tempfile
import unittest
import shutil
from cellmaps_pipeline.runner import ProgrammaticPipelineRunner
from cellmaps_pipeline.runner import CellmapsPipeline

SKIP_REASON = 'CELLMAPS_PIPELINE_INTEGRATION_TEST ' \
              'environment variable not set, cannot run integration ' \
              'tests'

@unittest.skipUnless(os.getenv('CELLMAPS_PIPELINE_INTEGRATION_TEST') is not None, SKIP_REASON)
class TestIntegrationCellmaps_pipeline(unittest.TestCase):
    """Tests for `cellmaps_pipeline` package."""

    def setUp(self):
        """Set up test fixtures, if any."""

    def tearDown(self):
        """Tear down test fixtures, if any."""

    def get_examples_dir(self):
        return os.path.join(os.path.dirname(os.path.abspath(os.path.dirname(__file__))), 'examples')

    def test_usage_example_takes_thirty_minutes(self):
        temp_dir = tempfile.mkdtemp()
        try:
            sys.stdout.write('Temp directory for test_usage_example_takes_thirty_minutes(): ' + str(temp_dir))
            sys.stdout.flush()
            # load the provenance as a dict
            with open(os.path.join('examples', 'provenance.json'), 'r') as f:
                json_prov = json.load(f)
            example_dir = self.get_examples_dir()
            outdir = os.path.join(temp_dir, 'myrun')
            runner = ProgrammaticPipelineRunner(outdir=outdir,
                                                samples=os.path.join(example_dir, 'samples.csv'),
                                                unique=os.path.join(example_dir, 'unique.csv'),
                                                edgelist=os.path.join(example_dir, 'edgelist.tsv'),
                                                baitlist=os.path.join(example_dir, 'baitlist.tsv'),
                                                provenance=json_prov,
                                                ppi_cutoffs=[0.001, 0.01],
                                                input_data_dict={})

            pipeline = CellmapsPipeline(outdir=outdir,
                                        runner=runner,
                                        input_data_dict={})
            self.assertEqual(0, pipeline.run())
        finally:
            sys.stdout.write('Removing temp directory for test_usage_example_takes_thirty_minutes(): ' + str(temp_dir))
            shutil.rmtree(temp_dir)



#! /usr/bin/env python

import argparse
import os.path
import sys
import logging
import logging.config
import json

from cellmaps_utils import logutils
from cellmaps_utils import constants
from cellmaps_imagedownloader.runner import CellmapsImageDownloader
from cellmaps_ppidownloader.runner import CellmapsPPIDownloader

from cellmaps_pipeline.exceptions import CellmapsPipelineError
from cellmaps_pipeline.runner import ProgrammaticPipelineRunner
from cellmaps_pipeline.runner import SLURMPipelineRunner
import cellmaps_pipeline
from cellmaps_pipeline.runner import CellmapsPipeline

logger = logging.getLogger(__name__)


def _parse_arguments(desc, args):
    """
    Parses command line arguments

    :param desc: description to display on command line
    :type desc: str
    :param args: command line arguments usually :py:func:`sys.argv[1:]`
    :type args: list
    :return: arguments parsed by :py:mod:`argparse`
    :rtype: :py:class:`argparse.Namespace`
    """
    parser = argparse.ArgumentParser(description=desc,
                                     formatter_class=constants.ArgParseFormatter)
    parser.add_argument('outdir', help='Output directory')
    parser.add_argument('--cm4ai_apms',
                        help='Path to CM4AI AP-MS table file in RO-Crate')
    parser.add_argument('--cm4ai_image',
                        help='Path to CM4AI IF Image table file in RO-Crate')
    parser.add_argument('--slurm', action='store_true',
                        help='If set, a main slurm script named slurm_cellmaps_job.sh will '
                             'be created in <outdir> along with other slurm scripts. '
                             'To run the pipeline directly invoke ./slurm_cellmaps_job.sh '
                             'on a SLURM submit node')
    parser.add_argument('--samples',
                        help='CSV file with list of IF images to download '
                             'in format of filename,if_plate_id,position,'
                             'sample,locations,antibody,ensembl_ids,'
                             'gene_names\n/archive/1/1_A1_1_,1,A1,1,'
                             'Golgi apparatus,HPA000992,ENSG00000066455,GOLGA5')
    parser.add_argument('--unique',
                        help='CSV file of unique samples '
                             'in format of:\n'
                             'antibody,ensembl_ids,gene_names,atlas_name,'
                             'locations,n_location\n'
                             'HPA040086,ENSG00000094914,AAAS,U-2 OS,'
                             'Nuclear membrane,1')
    parser.add_argument('--edgelist',
                        help='edgelist TSV file in format of:\n'
                             'GeneID1\tSymbol1\tGeneID2\tSymbol2\n'
                             '10159\tATP6AP2\t2\tA2M')
    parser.add_argument('--baitlist',
                        help='baitlist TSV file in format of:\n'
                             'GeneSymbol\tGeneID\t# Interactors\n'
                             '"ADA"\t"100"\t1.')
    parser.add_argument('--model_path', type=str,
                        default='https://github.com/CellProfiling/densenet/releases/download/v0.1.0/external_crop512_focal_slov_hardlog_class_densenet121_dropout_i768_aug2_5folds_fold0_final.pth',
                        help='URL or path to model file for image embedding')
    parser.add_argument('--fold', default=[1,2], nargs='+', type=int,
                        help='Image node attribute file fold(s) to use. '
                             'Each additional fold specified will create '
                             'additional 2.image_embedding_fold# and'
                             '3.coembedding_fold# directories that will be'
                             'fed into hierarchy step. Valid values are 1 or 2 or 1 2')
    parser.add_argument('--proteinatlasxml',
                        default='https://www.proteinatlas.org/download/proteinatlas.xml.gz',
                        help='URL or path to proteinatlas.xml or proteinatlas.xml.gz file '
                             'used to look for images not found in the standard location '
                             'on HPA')
    parser.add_argument('--ppi_cutoffs', nargs='+', type=float,
                        default=[0.001, 0.002, 0.003, 0.004, 0.005, 0.006,
                                 0.007, 0.008, 0.009, 0.01, 0.02, 0.03,
                                 0.04, 0.05, 0.10],
                        help='Cutoffs used to generate PPI input networks. For example, '
                             'a value of 0.1 means to generate PPI input network using the '
                             'top ten percent of coembedding entries. Each cutoff generates '
                             'another PPI network')
    parser.add_argument('--example_provenance', action='store_true',
                        help='If set, output example provenance to standard out and exit')
    parser.add_argument('--example_registered_provenance', action='store_true',
                        help='If set, output example provenance for FAIRSCAPE registered datasets')
    parser.add_argument('--provenance',
                        help='Path to file containing provenance '
                             'information about input files in JSON format. '
                             'This is required and not including will output '
                             'and error message with example of file')
    parser.add_argument('--fake', action='store_true',
                        help='If set, generate fake data for every step')
    parser.add_argument('--logconf', default=None,
                        help='Path to python logging configuration file in '
                             'this format: https://docs.python.org/3/library/'
                             'logging.config.html#logging-config-fileformat '
                             'Setting this overrides -v parameter which uses '
                             ' default logger. (default None)')
    parser.add_argument('--verbose', '-v', action='count', default=1,
                        help='Increases verbosity of logger to standard '
                             'error for log messages in this module. Messages are '
                             'output at these python logging levels '
                             '-v = WARNING, -vv = INFO, '
                             '-vvv = DEBUG, -vvvv = NOTSET (default ERROR '
                             'logging)')
    parser.add_argument('--version', action='version',
                        version=('%(prog)s ' +
                                 cellmaps_pipeline.__version__))

    return parser.parse_args(args)


def main(args):
    """
    Main entry point for program.
    The Cell Maps Pipeline takes ImmunoFluorescent images from the Human Protein
    Atlas along with Affinity Purification Mass Spectrometry data from one or
    more sources, converts them into embeddings that are then co-embedded and
    converted into an integrated interaction network from which a hierarchical
    model is derived.

    :param args: arguments passed to command line usually :py:func:`sys.argv[1:]`
    :type args: list

    :return: return value of :py:meth:`cellmaps_pipeline.runner.CellmapsPipeline.run`
             or ``2`` if an exception is raised
    :rtype: int
    """
    withguid = CellmapsPPIDownloader.get_example_provenance(with_ids=True)
    withguid.update(CellmapsImageDownloader.get_example_provenance(with_ids=True))

    register = CellmapsPPIDownloader.get_example_provenance()
    register.update(CellmapsImageDownloader.get_example_provenance())

    withguids_json = json.dumps(withguid, indent=2)
    register_json = json.dumps(register, indent=2)

    desc = """
Version {version}

The Cell Maps Pipeline takes ImmunoFluorescent images from the Human Protein
Atlas along with Affinity Purification Mass Spectrometry data from one or
more sources, converts them into embeddings that are then co-embedded and
converted into an integrated interaction network from which a hierarchical
model is derived.

The pipeline invokes six tools that each create an output directory where
results are stored and registered within Research Object Crates (RO-Crate)
using the FAIRSCAPE-cli.

For more information visit https://cellmaps-pipeline.readthedocs.io

                 ppi_download  image_download
                       ||            ||
                       \/            \/
              ppi_embedding  image_embedding_fold#
                        \\\\      //
                         \\\\    //
                    co_embedding_fold#
                           ||
                           \/
                        hierarchy
                           ||
                           \/
                      hierarchyeval

In default mode this will create the following directories under <outdir> specified
on the commandline:

  1.image_download
  1.ppi_download
  1.ppi_embedding
  2.image_embedding_fold1
  3.coembedding_fold1
  4.hierarchy
  4.hierarchyeval

If --fold 1 2 is passed in on the command line the directories would look like this:

  1.image_download
  1.ppi_download
  1.ppi_embedding
  2.image_embedding_fold1
  2.image_embedding_fold2
  3.coembedding_fold1
  3.coembedding_fold2
  4.hierarchy
  4.hierarchyeval


In addition, the --provenance flag is required and must be set to a path
to a JSON file.

If datasets are already registered with FAIRSCAPE then the following is sufficient:

{withguids}

If datasets are NOT registered, then the following is required:

{register}

Additional optional fields for registering datasets include
'url', 'used-by', 'associated-publication', and 'additional-documentation'
    """.format(version=cellmaps_pipeline.__version__,
               withguids=withguids_json,
               register=register_json)
    theargs = _parse_arguments(desc, args[1:])
    theargs.program = args[0]
    theargs.version = cellmaps_pipeline.__version__

    try:
        logutils.setup_cmd_logging(theargs)

        if theargs.example_provenance is True:
            logger.debug('Outputting example provenance to standard out')
            register = CellmapsPPIDownloader.get_example_provenance()
            register.update(CellmapsImageDownloader.get_example_provenance())
            sys.stdout.write(json.dumps(register, indent=2))
            sys.stdout.write('\n')
            sys.stdout.flush()
            return 0
        if theargs.example_registered_provenance is True:
            logger.debug('Outputting example provenance with registered '
                         'datasets to standard out')
            register = CellmapsPPIDownloader.get_example_provenance(with_ids=True)
            register.update(CellmapsImageDownloader.get_example_provenance(with_ids=True))
            sys.stdout.write(json.dumps(register, indent=2))
            sys.stdout.write('\n')
            sys.stdout.flush()
            return 0

        if theargs.provenance is None:
            sys.stderr.write('\n\n--provenance flag is required to run this tool. '
                             'Please pass '
                             'a path to a JSON file with the following data:\n\n')
            sys.stderr.write('If datasets are already registered with '
                             'FAIRSCAPE then the following is sufficient:\n\n')
            sys.stderr.write(withguids_json + '\n\n')
            sys.stderr.write('If datasets are NOT registered, then the following is required:\n\n')
            sys.stderr.write(register_json + '\n\n')
            return 1

        # load the provenance as a dict
        with open(theargs.provenance, 'r') as f:
            json_prov = json.load(f)

        if theargs.cm4ai_image is not None:
            if not os.path.exists(theargs.cm4ai_image) or not os.path.isfile(theargs.cm4ai_image):
                raise CellmapsPipelineError(f'Incorrect path was set for cm4ai_image. '
                                            f'File under path {theargs.cm4ai_image} does not exist.')
        if theargs.cm4ai_apms is not None:
            if not os.path.exists(theargs.cm4ai_apms) or not os.path.isfile(theargs.cm4ai_apms):
                raise CellmapsPipelineError(f'Incorrect path was set for cm4ai_apms. '
                                            f'File under path {theargs.cm4ai_apms} does not exist.')

        if theargs.slurm is True:
            runner = SLURMPipelineRunner(outdir=theargs.outdir,
                                         cm4ai_image=theargs.cm4ai_image,
                                         cm4ai_apms=theargs.cm4ai_apms,
                                         samples=theargs.samples,
                                         unique=theargs.unique,
                                         edgelist=theargs.edgelist,
                                         baitlist=theargs.baitlist,
                                         model_path=theargs.model_path,
                                         proteinatlasxml=theargs.proteinatlasxml,
                                         ppi_cutoffs=theargs.ppi_cutoffs,
                                         fake=theargs.fake,
                                         provenance=theargs.provenance,
                                         fold=theargs.fold,
                                         input_data_dict=theargs.__dict__)
        else:
            runner = ProgrammaticPipelineRunner(outdir=theargs.outdir,
                                                cm4ai_image=theargs.cm4ai_image,
                                                cm4ai_apms=theargs.cm4ai_apms,
                                                samples=theargs.samples,
                                                unique=theargs.unique,
                                                edgelist=theargs.edgelist,
                                                baitlist=theargs.baitlist,
                                                model_path=theargs.model_path,
                                                proteinatlasxml=theargs.proteinatlasxml,
                                                ppi_cutoffs=theargs.ppi_cutoffs,
                                                fake=theargs.fake,
                                                provenance=json_prov,
                                                fold=theargs.fold,
                                                input_data_dict=theargs.__dict__)

        return CellmapsPipeline(outdir=theargs.outdir,
                                runner=runner,
                                input_data_dict=theargs.__dict__).run()
    except Exception as e:
        logger.exception('Caught exception: ' + str(e))
        return 2
    finally:
        logging.shutdown()


if __name__ == '__main__':  # pragma: no cover
    sys.exit(main(sys.argv))



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""Tests for `cellmaps_pipeline` package."""


import unittest
import tempfile
import shutil
import os
from cellmaps_pipeline.runner import SLURMPipelineRunner
from cellmaps_pipeline.exceptions import CellmapsPipelineError


class TestSLURMPipelineRunner(unittest.TestCase):
    """Tests for `cellmaps_pipeline` package."""

    def setUp(self):
        """Set up test fixtures, if any."""

    def tearDown(self):
        """Tear down test fixtures, if any."""

    def test_constructor(self):
        """Tests constructor"""
        myobj = SLURMPipelineRunner('foo')
        self.assertIsNotNone(myobj)

    def test_write_slurm_directives(self):
        temp_dir = tempfile.mkdtemp()
        try:
            myobj = SLURMPipelineRunner(temp_dir)
            filename = os.path.join(temp_dir, 'foo.txt')
            with open(filename, 'w') as f:
                myobj._write_slurm_directives(out=f)
            with open(filename, 'r') as f:
                data = f.read().split('\n')
            self.assertEqual('#!/bin/bash', data[0])
        finally:
            shutil.rmtree(temp_dir)

    def test_generate_download_images_command(self):
        temp_dir = tempfile.mkdtemp()
        cm4ai_image = 'test_image'
        provenance = 'test_provenance'
        try:
            myobj = SLURMPipelineRunner(temp_dir, cm4ai_image=cm4ai_image, provenance=provenance)
            filename = os.path.join(temp_dir, 'imagedownloadjob.sh')
            myobj._generate_download_images_command()
            self.assertTrue(os.path.isfile(filename))
        finally:
            shutil.rmtree(temp_dir)

    def test_generate_download_ppi_command(self):
        temp_dir = tempfile.mkdtemp()
        edgelist = 'edgelist'
        baitlist = 'baitlist'
        provenance = 'test_provenance'
        try:
            myobj = SLURMPipelineRunner(temp_dir, edgelist=edgelist, baitlist=baitlist, provenance=provenance)
            filename = os.path.join(temp_dir, 'ppidownloadjob.sh')
            myobj._generate_download_ppi_command()
            self.assertTrue(os.path.isfile(filename))
        finally:
            shutil.rmtree(temp_dir)

    def test_generate_embed_image_command(self):
        temp_dir = tempfile.mkdtemp()
        try:
            myobj = SLURMPipelineRunner(temp_dir)
            filename = os.path.join(temp_dir, 'imageembedjob1.sh')
            myobj._generate_embed_image_command()
            self.assertTrue(os.path.isfile(filename))
        finally:
            shutil.rmtree(temp_dir)

    def test_generate_embed_ppi_command(self):
        temp_dir = tempfile.mkdtemp()
        try:
            myobj = SLURMPipelineRunner(temp_dir)
            filename = os.path.join(temp_dir, 'ppiembedjob.sh')
            myobj._generate_embed_ppi_command()
            self.assertTrue(os.path.isfile(filename))
        finally:
            shutil.rmtree(temp_dir)

    def test_generate_coembed_command(self):
        temp_dir = tempfile.mkdtemp()
        try:
            myobj = SLURMPipelineRunner(temp_dir)
            filename = os.path.join(temp_dir, 'coembeddingjob1.sh')
            myobj._generate_coembed_command(fold=1)
            self.assertTrue(os.path.isfile(filename))
        finally:
            shutil.rmtree(temp_dir)

    def test_generate_hierarchy_command(self):
        temp_dir = tempfile.mkdtemp()
        try:
            myobj = SLURMPipelineRunner(temp_dir)
            filename = os.path.join(temp_dir, 'hierarchyjob.sh')
            myobj._generate_hierarchy_command()
            self.assertTrue(os.path.isfile(filename))
        finally:
            shutil.rmtree(temp_dir)

    def test_slurm_run(self):
        temp_dir = tempfile.mkdtemp()
        samples = 'test_samples'
        unique = 'test_unique'
        edgelist = 'edgelist'
        baitlist = 'baitlist'
        provenance = 'test_provenance'
        try:
            myobj = SLURMPipelineRunner(temp_dir, samples=samples, unique=unique,
                                        edgelist=edgelist, baitlist=baitlist, provenance=provenance)
            filename = os.path.join(temp_dir, 'slurm_cellmaps_job.sh')
            myobj.run()
            self.assertTrue(os.path.isfile(filename))
        finally:
            shutil.rmtree(temp_dir)



# -*- coding: utf-8 -*-

"""Top-level package for Cell Maps Pipeline."""

__author__ = 'Cell Maps team'
__email__ = 'tools@cm4ai.org'
__version__ = '1.0.0'
__repo_url__ = 'https://github.com/idekerlab/cellmaps_pipeline'
__description__ = 'Runs full Cell Maps pipeline'



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""Tests for `cellmaps_pipeline` package."""

import os
import tempfile
import shutil

import unittest
from cellmaps_pipeline import cellmaps_pipelinecmd


class TestCellmaps_pipeline(unittest.TestCase):
    """Tests for `cellmaps_pipeline` package."""

    def setUp(self):
        """Set up test fixtures, if any."""

    def tearDown(self):
        """Tear down test fixtures, if any."""

    def test_parse_arguments(self):
        """Tests parse arguments"""
        res = cellmaps_pipelinecmd._parse_arguments('hi', ['outdir'])

        self.assertEqual(res.verbose, 1)
        self.assertEqual('outdir', res.outdir)
        self.assertEqual(res.logconf, None)

        someargs = ['-vv', '--logconf', 'hi', 'outdir']
        res = cellmaps_pipelinecmd._parse_arguments('hi', someargs)

        self.assertEqual(res.verbose, 3)
        self.assertEqual(res.logconf, 'hi')
        self.assertEqual('outdir', res.outdir)

    def test_main(self):
        """Tests main function"""

        # try where loading config is successful
        try:
            temp_dir = tempfile.mkdtemp()
            res = cellmaps_pipelinecmd.main(['myprog.py', 'outdir'])
            self.assertEqual(res, 1)
        finally:
            shutil.rmtree(temp_dir)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""Tests for `cellmaps_pipeline` package."""

import os
import unittest
from unittest.mock import MagicMock
import shutil
import tempfile
from cellmaps_pipeline.runner import CellmapsPipeline
from cellmaps_pipeline.exceptions import CellmapsPipelineError


class TestCellmapspipelinerunner(unittest.TestCase):
    """Tests for `cellmaps_pipeline` package."""

    def setUp(self):
        """Set up test fixtures, if any."""

    def tearDown(self):
        """Tear down test fixtures, if any."""

    def test_constructor(self):
        """Tests constructor"""
        myobj = CellmapsPipeline('foo')

        self.assertIsNotNone(myobj)

    def test_run_outdir_not_set(self):
        """ Tests run()"""
        try:
            myobj = CellmapsPipeline()
            myobj.run()
            self.fail('expected exception')
        except CellmapsPipelineError as e:
            self.assertEqual('outdir is None', str(e))

    def test_run_with_mockrunner_success(self):
        temp_dir = tempfile.mkdtemp()
        try:
            outdir = os.path.join(temp_dir, 'pipeline')
            runner = MagicMock()
            runner.run = MagicMock(return_value=0)
            myobj = CellmapsPipeline(outdir=outdir, runner=runner)
            self.assertEqual(0, myobj.run())

            self.assertTrue(os.path.isdir(outdir))
        finally:
            shutil.rmtree(temp_dir)
